#lang scheme

(require "structures.scm")
(require "parser.scm")

(define (test-expr check? sexpr) (test parse-expr check? sexpr))
(define (test-decl check? sexpr) (test parse-decl check? sexpr))

(define (test parse check? sexpr)
  (if (check? (parse sexpr))
      (printf "OK~n")
      (error 'test "failed: ~a~n" sexpr)))

; lit
(test-expr lit-expr? '1)
(test-expr lit-expr? #t)
(test-expr lit-expr? #f)
(test-expr lit-expr? '"hola")
(test-expr lit-expr? '(list a b c))
(test-expr lit-expr? '(quote (a b c)))

; vget & vset
(test-expr vget-expr? 'x)
(test-expr vset-expr? '(set! x v))

; let
(test-expr let-expr? '(let ((x 1)) x))
(test-expr let-expr? '(let ((x 1)) (notify "c") x))

; while
(test-expr while-expr? '(while (= x 1) x x))
(test-expr while-expr? '(while (= x 1) (printf "~a~n" x)))

; cond
(test-expr cond-expr? '(if c t e))
(test-expr cond-expr? '(if (send this even 13) t e))
(test-expr cond-expr? '(cond 
                         ((= 1 2) (error "1"))
                         ((= 2 1) (error "2"))
                         (else (printf "ok"))))

; prim-app
(test-expr prim-app-expr? '(+ 1 1))
(test-expr prim-app-expr? '(equal? 2 1))
(test-expr prim-app-expr? '(not (equal? 2 1)))
(test-expr prim-app-expr? '(printf "~a~n" (send (new Plus a b) eval)))

; new
(test-expr new-expr? '(new Object 1))

; fget
(test-expr fget-expr? '(fget x))

; fset
(test-expr fset-expr? '(fset! x v))

; send
(test-expr send-expr? '(send o foo 1 2 3))

; super
(test-expr super-expr? '(super foo 1 2 3))

; class
(test-decl class-decl? '(class B A (field f) (method m (a) 1)))
(test-decl class-decl? '(class B A (field f) (method m (a) (if c t e))))

; thread
(test-expr thread-expr? '(thread (new Object)))

; wait 
(test-expr wait-expr? '(wait "c"))

; notify
(test-expr notify-expr? '(notify "c"))

; deploy
(test-expr deploy-expr? 
           '(deploy ((#t #t #t (new Aspect))) "dummy" "dummy"))
(test-expr deploy-expr? 
           '(deploy ((#t #t #t (new Aspect)) (#t #t #t (new Aspect))) "dummy" "dummy"))